/*
 *
 *	Utility for setting user/group quota
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <linux/quota.h>

#include "common.h"
#include "quota_io.h"
#include "quotasys.h"

#define FL_COPY 1
#define FL_GRACE 2

struct dquot dquot;
char *mnt;
int type;
int flags, names;

ulong cvnum(char *from, char *err)
{
	char *errch;
	ulong ret;

	ret = strtol(from, &errch, 0);
	if (*errch)
		die(1, err);
	return ret;
}

void parse_options(int argcnt, char **argstr)
{
	int ret;
	char *slash = strrchr(argstr[0], '/');

	if (!slash)
		slash = argstr[0];
	else
		slash++;

	type = QS_USRQUOTA;
	while ((ret = getopt(argcnt, argstr, "Vughtp:")) != EOF) {
		switch (ret) {
			case '?':
			case 'h':
usage:
				printf("Utility for setting user/group quotas.\nUsage:\n%s [-u | -g] mountpoint (block-soft \
block-hard inode-soft inode-hard | -p model-name) name...\n", slash);
				printf("%s -t [-u | -g] mountpoint <block-time-limit> <inode-time-limit>\n", slash);
				printf("Bugs to %s\n", MY_EMAIL);
				exit(1);
			case 'V':
				version();
				exit(0);
			case 'g':
				type = QS_GRPQUOTA;
				break;
			case 'u':
				type = QS_USRQUOTA;
				break;
			case 't':
				flags |= FL_GRACE;
				break;
			case 'p':
				flags |= FL_COPY;
				if (type == QS_USRQUOTA)
					dquot.dq_id = name2uid(optarg);
				else
					dquot.dq_id = group2gid(optarg);
				break;
		}
	}
	if (flags & FL_COPY && flags & FL_GRACE) {
		fputs("Illegal combination of options.\n", stderr);
		goto usage;
	}
	if ((flags & FL_COPY && argcnt - optind < 2) || ((flags & FL_GRACE) && argcnt - optind != 3) || (!(flags & (FL_GRACE | FL_COPY)) && argcnt - optind < 6)) {
		fputs("Bad number of arguments.\n", stderr);
		goto usage;
	}
	mnt = argstr[optind];
	if (flags & FL_COPY)
		names = optind + 1;
	else if (flags & FL_GRACE) {
		dquot.dq_dqb.dqb_btime = cvnum(argstr[optind+1], "Illegal block grace time.\n");
		dquot.dq_dqb.dqb_itime = cvnum(argstr[optind+2], "Illegal inode grace time.\n");
	}
	else {
		dquot.dq_dqb.dqb_bsoftlimit = cvnum(argstr[optind+1], "Illegal block soft limit.\n");
		dquot.dq_dqb.dqb_bhardlimit = cvnum(argstr[optind+2], "Illegal block hard limit.\n");
		dquot.dq_dqb.dqb_isoftlimit = cvnum(argstr[optind+3], "Illegal inode soft limit.\n");
		dquot.dq_dqb.dqb_ihardlimit = cvnum(argstr[optind+4], "Illegal inode hard limit.\n");
		names = optind + 5;
	}
}

int main(int argcnt, char **argstr)
{
	struct dquot adquot;
	char dev_name[MAXDEVLEN];
	char mntpoint[MAXFILENAMELEN];

	parse_options(argcnt, argstr);
	init_mount(&mnt, 1);
	if (!next_mount(type, dev_name, mntpoint, NULL))
		die(1, "There's no such mountpoint.\n");
	free_mount();
	init_io(mnt, dev_name, type);
	if (flags & FL_GRACE) {
		info.dqi_bgrace = dquot.dq_dqb.dqb_btime;
		info.dqi_igrace = dquot.dq_dqb.dqb_itime;
		mark_quotafile_info_dirty();
	}
	else {
		if (flags & FL_COPY)
			read_dquot(dquot.dq_id, &dquot);
		for (; names < argcnt; names++) {
			if (type == QS_USRQUOTA)
				read_dquot(name2uid(argstr[names]), &adquot);
			else
				read_dquot(group2gid(argstr[names]), &adquot);
			adquot.dq_dqb.dqb_bsoftlimit = dquot.dq_dqb.dqb_bsoftlimit;
			adquot.dq_dqb.dqb_bhardlimit = dquot.dq_dqb.dqb_bhardlimit;
			adquot.dq_dqb.dqb_isoftlimit = dquot.dq_dqb.dqb_isoftlimit;
			adquot.dq_dqb.dqb_ihardlimit = dquot.dq_dqb.dqb_ihardlimit;
			update_times(&adquot.dq_dqb);
			commit_dquot(&adquot, 0);
		}
	}
	free_io();

	return 0;
}

